#include "DBHelpMySQL.h"

DBHelpMySQL::DBHelpMySQL()
{
	//配置登录信息
	char *tmp = getConnectionString();
	if ( tmp == NULL || strlen(tmp) == 0)
	{
		err_msg("getConnectionString() error");
		//设置默认连接字符串
		connectionString = new char[1024];
		memset(connectionString, 0, sizeof(connectionString));
		strcpy(connectionString, "localhost:3306:root:361254617:mysql");
	}
	if (!setConnectionInfo()) //配置失败则使用默认
	{
		err_msg("setConnectionString() error");
		server = "localhost";
		port = "3306";
		user = "root";
		pwd = "361254617";
		dbName = "mysql";
	}

	//初始化C API
	if (mysql_library_init(0, NULL, NULL) == 1)
	{
		err_msg("mysql_library_init() error");
		return;
	}
}

int DBHelpMySQL::setConnectionInfo()
{
	string info[5];
	char * buf;
	int i;
	buf = strtok(connectionString, ":");
	for(i = 0; buf != NULL; i++)
	{
		info[i] = buf;
		buf = strtok(NULL, ":");
	}
	if (i != 5)	//不匹配5个必须参数则失败
		return 0;
	server = info[0];
	port = info[1];
	user = info[2];
	pwd = info[3];
	dbName = info[4];
	return 1;
}

char * DBHelpMySQL::getConnectionString()
{
	FILE *fp;
	int connStrLength = 0;

	if (!(fp = fopen("config.lxm", "r+")))
	{
		err_msg("fopen() error");
		return NULL;
	}
	while(fgetc(fp) != EOF)
		connStrLength++;
	connectionString = new char[connStrLength+1];
	rewind(fp);
	return fgets(connectionString, connStrLength+1, fp);
}

DBHelpMySQL::~DBHelpMySQL(void)
{
	closeConnection();
	mysql_library_end();
	delete [] connectionString;
}

bool DBHelpMySQL::openConnection(bool toMult = false)
{
	if (connection != NULL)
		closeConnection();
	//初始化连接句柄
	if ((connection = mysql_init(NULL)) == NULL)
	{
		err_msg("mysql__init() error");
		return false;
	}
	if (toMult)
	{
		if (mysql_real_connect(connection, server.c_str(), user.c_str(), pwd.c_str(), dbName.c_str(), atoi(port.c_str()), NULL, CLIENT_MULTI_STATEMENTS) == NULL)
		{
			err_msg("mysql_real_connect() error");
			return false;
		}
	}
	else
	{
		if (mysql_real_connect(connection, server.c_str(), user.c_str(), pwd.c_str(), dbName.c_str(), atoi(port.c_str()), NULL, 0) == NULL)
		{
			err_msg("mysql_real_connect() error");

			return false;
		}
	}
	//支持中文
	mysql_query(connection, "SET NAMES GBK");
	return true;
}

void DBHelpMySQL::closeConnection()
{
	if (connection != NULL)
		 mysql_close(connection);
	connection = NULL;
}

bool DBHelpMySQL::buildPreProcess()
{
	if ((stmt = mysql_stmt_init(connection)) == NULL )
	{
		err_msg("mysql_stmt_init() error");
		return false;
	}
	return true;
}

bool DBHelpMySQL::closePreProcess()
{
	if (stmt != NULL && mysql_stmt_close(stmt))
	{
		err_msg("mysql_stmt_close() error");
		return false;
	}
	stmt = NULL;
	return true;
}

void DBHelpMySQL::occerror( const char *msg )
{
	err_msg(msg);
	closePreProcess();
	closeConnection();
}

MYSQL_STMT * DBHelpMySQL::execute(string cmdText, MYSQL_BIND *params, int paramsLength)
{
	int paramsNum = 0;
	if (!openConnection())
		return NULL;
	if (!buildPreProcess())
		return NULL;
	if (mysql_stmt_prepare(stmt, cmdText.c_str(), cmdText.size()))
	{
		occerror("mysql_stmt_prepare() error");

		return NULL;
	}
	//获取SQL参数数量
	paramsNum = mysql_stmt_param_count(stmt);
	//不匹配数量返回null
	if (paramsNum != paramsLength)
	{
		occerror("params not match");
		return NULL;
	}
	//绑定参数到SQL语句
	if (mysql_stmt_bind_param(stmt, params))
	{
		occerror("mysql_stmt_bind_param() error");
		return NULL;
	}
	//获取结果集元数据,以获取结果集的列数
	if ((prepare_meta_result = mysql_stmt_result_metadata(stmt)) == NULL )
	{
		occerror("mysql_stmt_result_metadata() error");
		return NULL;
	}
	DataRow.column_count= mysql_num_fields(prepare_meta_result);
	DataRow.columnNames = new char *[DataRow.column_count];
	if (mysql_stmt_execute(stmt))
	{
		occerror("mysql_stmt_execute() error");
		return NULL;
	}
	DataRow.mybind =  (MYSQL_BIND *)malloc(DataRow.column_count * sizeof(MYSQL_BIND));
	DataRow.dataArea = new void*[DataRow.column_count];
	DataRow.is_null = new my_bool[DataRow.column_count];
	DataRow.length = new unsigned long[DataRow.column_count];

	memset(DataRow.mybind, 0, DataRow.column_count*sizeof(MYSQL_BIND));
	for (unsigned int col = 0; col < DataRow.column_count; col++)
	{
		DataRow.mybind[col].buffer_type= mysql_fetch_field_direct(prepare_meta_result, col)->type;
		DataRow.dataArea[col] = malloc(DEFAULT_FIELD_SIZE);
		DataRow.mybind[col].buffer= (char *)DataRow.dataArea[col];
		DataRow.mybind[col].is_null= &DataRow.is_null[col];
		DataRow.mybind[col].length= &DataRow.length[col];
		DataRow.mybind[col].buffer_length= DEFAULT_FIELD_SIZE;
	}

	if (mysql_stmt_bind_result(stmt, DataRow.mybind))
	{
		occerror("mysql_stmt_bind_result() error");
		return NULL;
	}

	if (mysql_stmt_store_result(stmt))
	{
		occerror("mysql_stmt_store_result() error");
		return NULL;
	}
	DataRow.row_count = GetRowCount(stmt);
	for (int i = 0; i < (int) DataRow.column_count; i++)
	{
		DataRow.columnNames[i] = new char[strlen(prepare_meta_result->fields[i].name)+1];
		strcpy(DataRow.columnNames[i], prepare_meta_result->fields[i].name);
	}
	return stmt;
}

long DBHelpMySQL::ExecuteNonQuery(const string cmdText, MYSQL_BIND *params, int paramsLength)
{
	int rowsAffected = 0;
	int paramsNum = 0;


	if (!openConnection())
		return 0;
	if (!buildPreProcess())
		return 0;
	if (mysql_stmt_prepare(stmt, cmdText.c_str(), cmdText.size()))
	{
		occerror("mysql_stmt_prepare() error");
		closePreProcess();
		closeConnection();
		return rowsAffected;
	}
	paramsNum = mysql_stmt_param_count(stmt);
	if (paramsNum != paramsLength)
	{
		occerror("params not match");
		closePreProcess();
		closeConnection();
		return rowsAffected;
	}

	if (mysql_stmt_bind_param(stmt, params))
	{
		occerror("mysql_stmt_bind_param() error");
		closePreProcess();
		closeConnection();
		return rowsAffected;
	}

	if (mysql_stmt_execute(stmt))
	{
		occerror("mysql_stmt_execute() error");
		closePreProcess();
		closeConnection();
		return rowsAffected;
	}

	rowsAffected = mysql_stmt_affected_rows(stmt);

	FreeStmt(stmt);
	closePreProcess();
	closeConnection();
	return rowsAffected;
}

void* DBHelpMySQL::ExecuteScalar(const string cmdText, MYSQL_BIND *params, int paramsLength )
{
	if (execute(cmdText, params, paramsLength) == NULL)
	{
		occerror("execute() error");
		FreeStmt(stmt);
		Dispose();
	}

	if (mysql_stmt_fetch(stmt) != 0)
	{
		occerror("mysql_stmt_fetch() error");
		FreeStmt(stmt);
		Dispose();
	}

	if (DataRow.is_null[0])
	{
		FreeStmt(stmt);
		Dispose();
	}
	else
	{
		FreeStmt(stmt);
		return DataRow.dataArea[0];
	}
}

MYSQL_STMT * DBHelpMySQL::ExecuteReader(const string cmdText, MYSQL_BIND *params, int paramsLength )
{
	return execute(cmdText, params, paramsLength);
}

void DBHelpMySQL::Dispose()
{
	closePreProcess();
	closeConnection();
	delete [] DataRow.is_null;
	delete [] DataRow.length;
	for (int i = 0; i <(int) DataRow.column_count; i++)
	{
		free(DataRow.dataArea[i]);
		delete [] DataRow.columnNames[i];
	}
	free(DataRow.dataArea);
	delete [] DataRow.columnNames;
	delete [] DataRow.mybind;
	mysql_free_result(prepare_meta_result);
}

bool DBHelpMySQL::SelectDB( const string dbname )
{
	if (connection == NULL)
		if (!openConnection())
			return NULL;

	if (mysql_select_db(connection, dbname.c_str()) != 0)
	{
		occerror("mysql_select_db() error");
		return false;
	}
	else
	{
		dbName = dbname;
		closeConnection();
		return true;
	}
}

MYSQL_RES * DBHelpMySQL::Query( const string cmdText )
{
	if (connection == NULL)
		if (!openConnection())
			return NULL;
	if (mysql_real_query(connection, cmdText.c_str(), cmdText.size()) != 0)
	{
		occerror("mysql_real_query() error");
		return NULL;
	}
	return mysql_store_result(connection);
}

MYSQL * DBHelpMySQL::MultQuery( string cmdText )
{
	if (!openConnection(true))
		return NULL;
 	if (mysql_real_query(connection, cmdText.c_str(), cmdText.size()) != 0)
	{
		occerror("mysql_query() error");
		return NULL;
	}
	return connection;
}

MYSQL_ROW DBHelpMySQL::GetNextRecord( MYSQL_RES * dataset )
{
	if (dataset != NULL)
		return mysql_fetch_row(dataset);
	return NULL;
}

DATAROW& DBHelpMySQL::GetNextRecord( MYSQL_STMT * stmt )
{
	mysql_stmt_fetch(stmt);
	return DataRow;
}

MYSQL_FIELD* DBHelpMySQL::GetFieldInfoOfIndex( MYSQL_RES * dataset, unsigned int index )
{
	if (dataset == NULL)
		return NULL;
	if ( index >= dataset->field_count)
		return NULL;
	return mysql_fetch_field_direct(dataset, index);
}

void DBHelpMySQL::GetState(char *stat)
{
	if (connection == NULL)
		if (!openConnection())
			return ;
	strcpy(stat, mysql_stat(connection));
	closeConnection();
}

void DBHelpMySQL::FreeResult( MYSQL_RES * dataset )
{
	if (dataset != NULL)
		mysql_free_result(dataset);
	else if (prepare_meta_result != NULL)
		mysql_free_result(prepare_meta_result);
}

void DBHelpMySQL::GetServerVerion( char *version )
{
	if (connection == NULL)
		if (!openConnection())
			return ;
	strcpy(version, mysql_get_server_info(connection));
	closeConnection();
}

unsigned int DBHelpMySQL::GetProtocolInfo()
{
	if (connection == NULL)
		if (!openConnection())
			return 0;
	unsigned int t = mysql_get_proto_info(connection);
	closeConnection();
	return t;
}

void DBHelpMySQL::GetHostInfo( char *hostInfo )
{
	if (connection == NULL)
		if (!openConnection())
			return ;
	strcpy(hostInfo, mysql_get_host_info(connection));
	closeConnection();
}

const char* DBHelpMySQL::GetClientInfo()
{
	return mysql_get_client_info();
}

my_ulonglong DBHelpMySQL::GetRowCount( MYSQL_RES * dataset )
{
	if (dataset != NULL)
		return mysql_num_rows(dataset);
	return 0;
}

my_ulonglong DBHelpMySQL::GetRowCount( MYSQL_STMT * stmt )
{
	if (stmt != NULL)
		return mysql_stmt_num_rows(stmt);
	return 0;
}

my_ulonglong DBHelpMySQL::GetColumnCount( MYSQL_RES * dataset )
{
	if (dataset != NULL)
		return mysql_num_fields(dataset);
	return 0;
}

my_ulonglong DBHelpMySQL::GetColumnCount( MYSQL_STMT * stmt )
{
	if (stmt != NULL)
		return stmt->field_count;
	return 0;
}

void DBHelpMySQL::SeekRecord( MYSQL_RES * dataset, my_ulonglong index )
{
	if (dataset == NULL)
		return;
	if (index >= dataset->field_count)
		return;
	mysql_data_seek(dataset, index);
}

void DBHelpMySQL::SeekRecord( MYSQL_STMT * stmt , my_ulonglong index )
{
	if (stmt == NULL)
		return;
	if (index >= stmt->field_count)
		return;
	mysql_stmt_data_seek(stmt, index);
}

my_ulonglong DBHelpMySQL::GetAffectedCount( MYSQL * conn )
{
	if (conn == NULL)
		return -1;
	return mysql_affected_rows(conn);
}

MYSQL_RES * DBHelpMySQL::GetResultSet( MYSQL * conn )
{
	if (conn == NULL)
		return NULL;
	return mysql_store_result(conn);
}

int DBHelpMySQL::GetNextResultSet( MYSQL * conn )
{
	if (conn == NULL)
		return 1;
	return mysql_next_result(conn);
}

void DBHelpMySQL::FreeConn()
{
	closeConnection();
}

bool DBHelpMySQL::HasMoreResult( MYSQL * conn )
{
	if (conn == NULL)
		return false;
	return mysql_more_results(conn);
}

my_ulonglong DBHelpMySQL::GetAutoID()
{
	my_ulonglong used_id = 0;
	MYSQL_RES *r;
	if (connection == NULL)
		return 0;
	if ((r = mysql_store_result(connection)) == 0 &&
		mysql_field_count(connection) == 0 &&
		mysql_insert_id(connection) != 0)
	{
		used_id = mysql_insert_id(connection);
	}
	if (r != NULL)
		mysql_free_result(r);;
	FreeConn();
	return used_id;

}

bool DBHelpMySQL::IsConnected()
{
	if (connection == NULL)
		return false;
	return mysql_ping(connection) == 0;
}

bool DBHelpMySQL::ShutDown()
{
	if (!IsConnected())
		openConnection();
	bool flag = mysql_shutdown(connection, SHUTDOWN_DEFAULT) == 0;
	closeConnection();
	return flag;
}

void DBHelpMySQL::err_msg( const char *fmt, ... )
{
	va_list     ap;
	va_start(ap, fmt);
	err_doit(1, fmt, ap);
	va_end(ap);
	return;
}

void DBHelpMySQL::err_doit(int errnoflag, const char *fmt, va_list ap)
{
	int		errno_save, n;
	char	buf[MAXLINE + 1];

	errno_save = errno;

	vsnprintf(buf, MAXLINE, fmt, ap);

	n = strlen(buf);
	if (errnoflag)
	{
		if (errno_save != 0)
			sprintf(buf + n, ": %s", strerror(errno_save));
		else
		{
			if (connection != NULL)
				sprintf(buf + n, ": %s", mysql_error(connection));
			else if (stmt != NULL)
				sprintf(buf + n, ": %s", mysql_stmt_error(stmt));
			else
				sprintf(buf + n, ": %s", strerror(errno_save));
		}
	}
	strcat(buf, "\n");

	fflush(stdout);
	fputs(buf, stderr);
	fflush(stderr);

	return;
}

bool DBHelpMySQL::StartTransaction()
{
	bool flag = ExecuteSQL("START TRANSACTION;");
	if(!flag)
		err_msg("START TRANSACTION error");
	return flag;
}

bool DBHelpMySQL::CommitTransaction()
{
	if(connection == NULL)
	return false;
	bool flag = mysql_commit(connection) == 0;
	if(!flag)
	err_msg("mysql_commit() error");
	return flag;
	//return ExecuteSQL("COMMIT;");
}

bool DBHelpMySQL::RollBackTransaction()
{
	//return ExecuteSQL("ROLLBACK;");
	if(connection == NULL)
		return false;
	bool flag = mysql_rollback(connection) == 0;
	if(!flag)
		err_msg("mysql_rollback() error");
	return flag;
}

bool DBHelpMySQL::SetAutoCommit(bool isAuto)
{
	if(isAuto)
		return ExecuteSQL("SET AUTOCOMMIT=1");
	else
		return ExecuteSQL("SET AUTOCOMMIT=0");
}

MYSQL* DBHelpMySQL::GetMYSQL(bool isMutl=false)
{
	if (connection == NULL)
	{
		if(isMutl)
			openConnection(true);
		else
			openConnection();
	}
	return connection;
}

bool DBHelpMySQL::ExecuteSQL(const string cmdText)
{
	if (connection == NULL)
		if (!openConnection())
			return false;
	if (mysql_real_query(connection, cmdText.c_str(), cmdText.size()) != 0)
		return false;
	return true;
}

const char * DBHelpMySQL::User( const char * u)
{
	if (u != NULL)
		user = u;
	return user.c_str();
}

const char * DBHelpMySQL::Pwd( const char * p)
{
	if (p != NULL)
		pwd = p;
	return pwd.c_str();
}

const char * DBHelpMySQL::Server( const char * s)
{
	if (s != NULL)
		server = s;
	return server.c_str();
}

const char * DBHelpMySQL::Port( const char * p)
{
	if (p != NULL)
		port = p;
	return port.c_str();
}

const char * DBHelpMySQL::DataBaseName( const char * d)
{
	if (d != NULL)
		dbName = d;
	return dbName.c_str();
}

bool DBHelpMySQL::SetOption(enum mysql_option option, const char *arg )
{
	if (IsConnected())
		closeConnection();
	bool flag = mysql_options(connection, option, arg) == 0;
	if (!flag)
		err_msg("mysql_options() error");
	return flag;
}

void DBHelpMySQL::FreeStmt( MYSQL_STMT * s )
{
	if(s != NULL)
		mysql_stmt_free_result(s);
	else if(stmt != NULL)
		mysql_stmt_free_result(stmt);
}


MYSQL_STMT * DBHelpMySQL::stmt;
MYSQL * DBHelpMySQL::connection;

std::string DBHelpMySQL::port;
string DBHelpMySQL::server;
string DBHelpMySQL::user;
string DBHelpMySQL::pwd;
string DBHelpMySQL::dbName;
char* DBHelpMySQL::connectionString;


MYSQL_RES *DBHelpMySQL::prepare_meta_result;
DATAROW DBHelpMySQL::DataRow;

