#ifndef DBHELPMYSQL_H
#define DBHELPMYSQL_H

//#include </usr/include/mysql/mysql.h>
#include <mysql.h>
#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <stdarg.h>

using namespace std;

#define DEFAULT_FIELD_SIZE 64
#define MAXLINE 4096

typedef struct datarow
{
	MYSQL_BIND *mybind;
	unsigned int column_count;
	unsigned int row_count;
	char **columnNames;
	unsigned long *length;
	my_bool       *is_null;
	void **dataArea;
}DATAROW;

class DBHelpMySQL
{
public:
	~DBHelpMySQL(void);
	DBHelpMySQL();
private:
	DBHelpMySQL& operator=(const DBHelpMySQL& db);
	DBHelpMySQL(const DBHelpMySQL& db);
	//连接字符串
	static char* connectionString;
	//服务器名
	static string server;
	//port
	static string port;
	//登录名
	static string user;
	//密码
	static string pwd;
	//数据库名
	static string dbName;
	//字段元数据结构
	static MYSQL_RES *prepare_meta_result;

protected:
	//Connection对象
	static MYSQL *connection;
	//MYSQL_STMT预处理语句对象
	static MYSQL_STMT *stmt;
	//结果集对象
	static DATAROW DataRow;

private:
	//读取config.lxm获得连接字符串
	char * getConnectionString();
	//解析连接字符串,为登录赋值
	int setConnectionInfo();
	//公用方法,用于执行带参数SQL查询.
	MYSQL_STMT * execute(string cmdText, MYSQL_BIND *params, int paramsLength);
	//当发生错误时,显示错误信息并释放资源
	static void occerror(const char *msg);
	//打开连接,默认不支持多语句查询,参数true表示支持多语句查询
	static bool openConnection(bool);
	//关闭服务器连接
	static void closeConnection();
	//开启预处理
	static bool buildPreProcess();
	//关闭预处理
	static bool closePreProcess();
	static void err_msg(const char *fmt, ...);
	static void err_doit(int errnoflag, const char *fmt, va_list ap);
public:
	//************************************
	// Method:    释放资源,并关闭服务器连接和关闭预处理
	// FullName:  DBHelpMySQL::Dispose
	// Access:    public
	// Returns:   void
	//************************************
	void Dispose();

	//************************************
	// Method:    执行SQL语句返回受影响的行数
	// FullName:  DBHelpMySQL::ExecuteNonQuery
	// Access:    public
	// Returns:   受影响的行数
	// Qualifier: params参数应该调用memset以清空内存
	// Parameter: const string cmdText SQL语句
	// Parameter: MYSQL_BIND * params 参数对象数组,可以为NULL
	// Parameter: int paramsLength 参数数组的长度,当params是NULL时,本参数应该为0
	//************************************
	long ExecuteNonQuery(const string cmdText, MYSQL_BIND *params, int paramsLength);

	//************************************
	// Method:    执行SQL语句返回第一行第一列
	// FullName:  DBHelpMySQL::ExecuteScalar
	// Access:    public
	// Returns:   第一行第一列的值,没有或失败将返回NULL
	// Qualifier: 使用后应该调用Dispose()
	// Parameter: const string cmdText SQL语句
	// Parameter: MYSQL_BIND * params 参数对象数组
	// Parameter: int paramsLength 参数数组的长度
	//************************************
	void * ExecuteScalar(const string cmdText, MYSQL_BIND *params, int paramsLength);

	//************************************
	// Method:    执行SQL语句返回数据结果行数组
	// FullName:  DBHelpMySQL::ExecuteReader
	// Access:    public
	// Returns:   数据结果行数组,没有或失败将返回NULL
	// Qualifier: 不使用时应该调用FreeStmt和Dispose函数
	// Parameter: const string cmdText SQL语句
	// Parameter: MYSQL_BIND * params 参数对象数组
	// Parameter: int paramsLength 参数数组的长度
	//************************************
	MYSQL_STMT * ExecuteReader(const string cmdText, MYSQL_BIND *params, int paramsLength);

	//************************************
	// Method:    执行SQL语句
	// FullName:  DBHelpMySQL::Query
	// Access:    public
	// Returns:   结果集对象,没有或失败将返回NULL
	// Qualifier: 使用后应该调用FreeResult(MYSQL_RES*)函数和FreeConn函数释放资源
	// Parameter: const string cmdText SQL语句
	//************************************
	MYSQL_RES * Query(const string cmdText);

	//************************************
	// Method:    获取自动增长字段的ID
	// FullName:  DBHelpMySQL::GetAutoID
	// Access:    public
	// Returns:   刚插入的id,失败返回0
	// Qualifier: 获得最后插入的第一行数据；即使你一次性插大批数据也是只返回该批次第一行自增后的值.
	//			  使用后应该调用FreeConn函数.
	//************************************
	my_ulonglong GetAutoID();

	//************************************
	// Method:    执行多语句查询
	// FullName:  DBHelpMySQL::MultQuery
	// Access:    public
	// Returns:   执行后的MYSQL *对象,没有或失败将返回NULL
	// Qualifier: 使用后应该调用FreeResult和FreeConn函数断开连接
	// Parameter: string cmdText SQL语句
	//************************************
	MYSQL * MultQuery(string cmdText);

	//************************************
	// Method:    获取结果集
	// FullName:  DBHelpMySQL::GetResultSet
	// Access:    public
	// Returns:   MYSQL_RES *结果集对象,失败返回NULL
	// Qualifier: 使用后应该调用FreeResult(MYSQL_RES*)函数释放结果集
	// Parameter: MYSQL * conn 连接对象
	//************************************
	MYSQL_RES * GetResultSet(MYSQL * conn);

	//************************************
	// Method:    获取下一个结果集
	// FullName:  DBHelpMySQL::GetNextResultSet
	// Access:    public
	// Returns:   0成功,有多个结果; -1成功,但没有多个结果; >0出错
	// Qualifier: 参考MultQuery的说明
	// Parameter: MYSQL * conn 连接对象
	//************************************
	int GetNextResultSet(MYSQL * conn);

	//************************************
	// Method:    判断是否有下一个结果集
	// FullName:  DBHelpMySQL::HasMoreResult
	// Access:    public
	// Returns:   bool true表示有
	// Qualifier: 参考MultQuery的说明
	// Parameter: MYSQL * conn 连接对象
	//************************************
	bool HasMoreResult(MYSQL * conn);

	//************************************
	// Method:    返回受影响的行数
	// FullName:  DBHelpMySQL::GetAffectedCount
	// Access:    public
	// Returns:   受影响的行数,失败返回-1
	// Qualifier: 参考MultQuery的说明
	// Parameter: MYSQL * conn 连接对象
	//************************************
	my_ulonglong GetAffectedCount(MYSQL * conn);

	//************************************
	// Method:    返回结果集中的行数
	// FullName:  DBHelpMySQL::GetRowCount
	// Access:    public
	// Returns:   行数
	// Qualifier: 似乎不能和GetColumnCount函数同时运用于逗号运算符
	// Parameter: MYSQL_RES * dataset 要计算的结果集
	//************************************
	my_ulonglong GetRowCount(MYSQL_RES * dataset);

	//************************************
	// Method:    返回结果集中的行数
	// FullName:  DBHelpMySQL::GetRowCount
	// Access:    public
	// Returns:   行数
	// Qualifier: 似乎不能和GetColumnCount函数同时运用于逗号运算符
	// Parameter: MYSQL_STMT * stmt 要计算的结果集
	//************************************
	my_ulonglong GetRowCount(MYSQL_STMT * stmt);

	//************************************
	// Method:    返回结果集中的列数
	// FullName:  DBHelpMySQL::GetColumnCount
	// Access:    public
	// Returns:   列数
	// Qualifier: 似乎不能和GetRowCount函数同时运用于逗号运算符
	// Parameter: MYSQL_RES * dataset 要计算的结果集
	//************************************
	my_ulonglong GetColumnCount(MYSQL_RES * dataset);

	//************************************
	// Method:    返回结果集中的列数
	// FullName:  DBHelpMySQL::GetColumnCount
	// Access:    public
	// Returns:   列数
	// Qualifier: 似乎不能和GetRowCount函数同时运用于逗号运算符
	// Parameter: MYSQL_STMT * stmt 要计算的结果集
	//************************************
	my_ulonglong GetColumnCount( MYSQL_STMT * stmt );

	//************************************
	// Method:    返回结果集的下一条记录行
	// FullName:  DBHelpMySQL::GetNextRecord
	// Access:    public
	// Qualifier: 不使用时应该调用FreeResult函数
	// Returns:   MYSQL_ROW 行对象,没有或失败将返回NULL
	// Parameter: MYSQL_RES * dataset 要计算的结果集
	//************************************
	MYSQL_ROW GetNextRecord(MYSQL_RES * dataset);

	//************************************
	// Method:    返回结果集的下一条记录行
	// FullName:  DBHelpMySQL::GetNextRecord
	// Access:    public
	// Returns:   DATAROW 结果行
	// Qualifier: 不使用应该调用FreeStmt和Dispose()
	// Parameter: MYSQL_STMT * stmt 要计算的结果集
	//************************************
	DATAROW& GetNextRecord( MYSQL_STMT * stmt );

	//************************************
	// Method:    定位到结果集的某一行
	// FullName:  DBHelpMySQL::SeekRecord
	// Access:    public
	// Parameter: MYSQL_RES * dataset 要计算的结果集,没有或失败将返回NULL
	// Parameter: my_ulonglong index 要定位的行索引,从0开始
	//************************************
	void SeekRecord(MYSQL_RES * dataset, my_ulonglong index);

	//************************************
	// Method:    定位到结果集的某一行
	// FullName:  DBHelpMySQL::SeekRecord
	// Access:    public
	// Parameter: MYSQL_STMT * stmt 要计算的结果集,没有或失败将返回NULL
	// Parameter: my_ulonglong index 要定位的行索引,从0开始
	//************************************
	void SeekRecord(MYSQL_STMT * stmt , my_ulonglong index);

	//************************************
	// Method:    获取指定索引的字段信息
	// FullName:  DBHelpMySQL::GetFieldInfoOfIndex
	// Access:    public
	// Returns:   MYSQL_FIELD* 字段类型信息,没有或失败将返回NULL
	// Parameter: MYSQL_RES * dataset 要计算的结果集
	// Parameter: unsigned int index 要定位的索引,从0开始
	//************************************
	MYSQL_FIELD* GetFieldInfoOfIndex(MYSQL_RES * dataset, unsigned int index);



	//************************************
	// Method:    释放结果集
	// FullName:  DBHelpMySQL::FreeResult
	// Access:    public
	// Qualifier: 如果参数为NULL,则释放内部对象
	// Parameter: MYSQL_RES * dataset 要释放的结果集
	//************************************
	void FreeResult(MYSQL_RES * dataset);

	//************************************
	// Method:    释放连接对象,关闭连接
	// FullName:  DBHelpMySQL::FreeConn
	// Access:    public
	//************************************
	void FreeConn();

	//************************************
	// Method:    改变数据库
	// FullName:  DBHelpMySQL::SelectDB
	// Access:    public
	// Returns:   bool true表示成功
	// Qualifier: 配置文件必须存在(config.xml)
	// Parameter: const string dbName 数据库名
	//************************************
	bool SelectDB(const string dbName);

	//************************************
	// Method:    返回数据库信息
	// FullName:  DBHelpMySQL::GetState
	// Access:    public
	// Parameter: char * stat 结果参数,存放信息
	//************************************
	void GetState(char *stat);

	//************************************
	// Method:    返回服务器版本信息
	// FullName:  DBHelpMySQL::GetServerVerion
	// Access:    public
	// Parameter: char * version 结果参数,存放信息
	//************************************
	void GetServerVerion(char *version);

	//************************************
	// Method:    返回协议版本号
	// FullName:  DBHelpMySQL::GetProtocolInfo
	// Access:    public
	// Returns:   unsigned int 协议版本号
	//************************************
	unsigned int GetProtocolInfo();

	//************************************
	// Method:    返回描述连接主机的信息
	// FullName:  DBHelpMySQL::GetHostInfo
	// Access:    public
	// Parameter: char * hostInfo 结果参数,存放信息
	//************************************
	void GetHostInfo(char *hostInfo);

	//************************************
	// Method:    返回客户端信息
	// FullName:  DBHelpMySQL::GetClientInfo
	// Access:    public
	// Returns:   const char* 客户信息
	//************************************
   const char* GetClientInfo();

	//************************************
	// Method:    判断是否已建立连接
	// FullName:  DBHelpMySQL::IsConnected
	// Access:    public
	// Returns:   bool true表示已连接
	//************************************
	bool IsConnected();

	//************************************
	// Method:    关闭MYSQL服务器
	// FullName:  DBHelpMySQL::ShutDown
	// Access:    public
	// Returns:   bool true已关闭
	//************************************
	bool ShutDown();

	//************************************
	// Method:    开始事务
	// FullName:  DBHelpMySQL::StartTransaction
	// Access:    public
	// Returns:   bool true表示启动成功
	// Qualifier: 它会建立服务器连接,因此事务结束后应该调用FreeConn()函数
	//************************************
	bool StartTransaction();

	//************************************
	// Method:    提交事务
	// FullName:  DBHelpMySQL::CommitTransaction
	// Access:    public
	// Returns:   bool true表示成功
	// Qualifier: 此时服务器连接已建立,因此事务结束后应该调用FreeConn()函数
	//************************************
	bool CommitTransaction();

	//************************************
	// Method:    RollBackTransaction
	// FullName:  DBHelpMySQL::RollBackTransaction
	// Access:    public
	// Returns:   bool true表示成功
	// Qualifier: 此时服务器连接已建立,因此事务结束后应该调用FreeConn()函数
	//************************************
	bool RollBackTransaction();

	//************************************
	// Method:    设置自动提交,系统默认为自动
	// FullName:  DBHelpMySQL::SetAutoCommit
	// Access:    public
	// Returns:   bool true自动 false手动
	// Qualifier: 它会建立服务器连接,因此本连接不再使用后应该调用FreeConn()函数
	// Parameter: bool isAuto
	//************************************
	bool SetAutoCommit(bool isAuto);

	//************************************
	// Method:    返回已建立连接的MYSQL连接对象
	// FullName:  DBHelpMySQL::GetMYSQL
	// Access:    public
	// Returns:   MYSQL*对象
	// Qualifier: 不使用应该调用FreeConn()函数
	// Parameter: bool true表示返回支持多语句查询的MYSQL*对象
	//************************************
	MYSQL* GetMYSQL(bool );

	//************************************
	// Method:    执行SQL语句
	// FullName:  DBHelpMySQL::ExecuteSQL
	// Access:    public
	// Returns:   bool true表示执行成功
	// Qualifier: 它会建立服务器连接,因此本连接不再使用后应该调用FreeConn()函数
	// Parameter: const string cmdText SQL语句
	//************************************
	bool ExecuteSQL(const string cmdText);

	//************************************
	// Method:    获取或设置用户名
	// FullName:  DBHelpMySQL::User
	// Access:    public
	// Returns:   用户名
	// Parameter: 新的用户名,如果为NULL则仅返回用户名
	//************************************
	const char * User(const char *u  = NULL);

	//************************************
	// Method:    获取或设置密码
	// FullName:  DBHelpMySQL::Pwd
	// Access:    public
	// Returns:   密码
	// Parameter: 新的密码,如果为NULL则仅返回密码
	//************************************
	const char * Pwd(const char * p = NULL);

	//************************************
	// Method:    获取或设置主机名
	// FullName:  DBHelpMySQL::Server
	// Access:    public
	// Returns:   主机名
	// Parameter: 新的主机名,如果为NULL则仅返回主机名
	//************************************
	const char * Server(const char *s = NULL);

	//************************************
	// Method:    获取或设置端口号
	// FullName:  DBHelpMySQL::Port
	// Access:    public
	// Returns:   端口号
	// Parameter: 新的端口号,如果为NULL则仅返回密码
	//************************************
	const char * Port(const char *p = NULL);

	//************************************
	// Method:    获取或设置数据库名
	// FullName:  DBHelpMySQL::DataBaseName
	// Access:    public
	// Returns:   数据库名
	// Parameter: 新的数据库名,如果为NULL则仅返回数据库名
	//************************************
	const char * DataBaseName(const char *d = NULL);

	//************************************
	// Method:    设置额外的连接选项
	// FullName:  DBHelpMySQL::SetOption
	// Access:    public
	// Returns:   bool true表示配置成功
	// Qualifier: 如果已连接则此函数会关闭连接,保证在配置在正确的前提下执行(即未连接下才能成功)
	// Parameter: enum mysql_option option
	// Parameter: const char * arg
	//************************************
    bool SetOption(enum mysql_option option, const char *arg);

	//************************************
	// Method:    释放结果集
	// FullName:  DBHelpMySQL::FreeStmt
	// Access:    public
	// Returns:   void
	// Qualifier: 如果参数为NULL,则释放内部对象
	// Parameter: MYSQL_STMT * s 要释放的结果集
	//************************************
	void FreeStmt( MYSQL_STMT * s );
};

#endif
