﻿drop database if exists refactoring;

CREATE DATABASE refactoring;
USE refactoring;

CREATE TABLE area_accounts
(
  areaid    int(11)    NOT NULL,
  accountid bigint(20) NOT NULL,
  PRIMARY KEY (areaid, accountid)
);

CREATE TABLE transactions
(
  txid      bigint(20)  NOT NULL  PRIMARY KEY,
  accountid bigint(20)  NOT NULL,
  txdate    datetime    NOT NULL,
  amount    float       NOT NULL,
  curr      char(3)     NOT NULL
)
ENGINE=InnoDB;

CREATE TABLE check_log
(
  txid        bigint(20)  NOT NULL  PRIMARY KEY,
  conv_amount float       NOT NULL
)
ENGINE=InnoDB;

CREATE TABLE currencies
(
  iso           char(3)     NOT NULL PRIMARY KEY,
  currency_name varchar(64) NOT NULL
)
ENGINE=InnoDB;

CREATE TABLE currency_rates
(
  iso       char(3)  NOT NULL,
  rate      float    NOT NULL,
  rate_date date     NOT NULL,
  PRIMARY KEY(iso, rate_date)
)
ENGINE=InnoDB;

CREATE TABLE currency_weight
(
  iso       char(3)  NOT NULL PRIMARY KEY,
  weight    float    NOT NULL,
  CONSTRAINT FK_W_C FOREIGN KEY(iso) REFERENCES currencies(iso)
)
ENGINE=InnoDB;

CREATE TABLE thresholds
(
  iso         char(3)  NOT NULL  PRIMARY KEY,
  threshold   float    NOT NULL,
  CONSTRAINT FK_T1_C FOREIGN KEY(iso) REFERENCES currencies(iso)
)
ENGINE=InnoDB;


INSERT INTO currencies (currency_name, iso) VALUES ('Afghanistan Afghani', 'AFA'),
 ('Albanian Lek', 'ALL'),
 ('Algerian Dinar', 'DZD'),
 ('Andorran Franc', 'ADF'),
 ('Andorran Peseta', 'ADP'),
 ('Angolan Kwanza', 'AOA'),
 ('Argentine Austral', 'ARA'),
 ('Argentine Peso', 'ARS'),
 ('Armenian Dram', 'AMD'),
 ('Aruban Florin', 'AWG'),
 ('Australian Dollar', 'AUD'),
 ('Azerbaijan Manat', 'AZM'),
 ('Bahamian Dollar', 'BSD'),
 ('Bahraini Dinar', 'BHD'),
 ('Bangladeshi Taka', 'BDT'),
 ('Barbados Dollar', 'BBD'),
 ('Belarusian Ruble', 'BYR'),
 ('Belize Dollar', 'BZD'),
 ('CFA Franc BCEAO', 'XOF'),
 ('Bermudian Dollar', 'BMD'),
 ('Bhutan Ngultrum', 'BTN'),
 ('Bolivian Boliviano', 'BOB'),
 ('Bosnia and Herzegovina Convertible Mark', 'BAM'),
 ('Botswana Pula', 'BWP'),
 ('Brazilian Cruzeiro', 'BRC'),
 ('Brazilian Real', 'BRL'),
 ('Brunei Dollar', 'BND'),
 ('Bulgarian Lev', 'BGN'),
 ('Burundi Franc', 'BIF'),
 ('CFA Franc BEAC', 'XAF'),
 ('Kampuchean Riel', 'KHR'),
 ('Canadian Dollar', 'CAD'),
 ('Cape Verde Escudo', 'CVE'),
 ('Cayman Islands Dollar', 'KYD'),
 ('Chilean Peso', 'CLP'),
 ('Chinese Yuan Renminbi', 'CNY'),
 ('Colombian Peso', 'COP'),
 ('Comoros Franc', 'KMF'),
 ('Congolese Franc', 'CDF'),
 ('Costa Rican Colon', 'CRC'),
 ('Croatian Kuna', 'HRK'),
 ('Cuban Peso', 'CUP'),
 ('Cyprus Pound', 'CYP'),
 ('Czech Koruna', 'CZK'),
 ('Danish Krone', 'DKK'),
 ('Djibouti Franc', 'DJF'),
 ('Dominican Peso', 'DOP'),
 ('Euro', 'EUR'),
 ('Ecuador Sucre', 'ECS'),
 ('Egyption Pound', 'EGP'),
 ('El Salvador Colon', 'SVC'),
 ('Estonian Kroon', 'EEK'),
 ('Ethiopian Birr', 'ETB'),
 ('Falkland Islands Pound', 'FKP'),
 ('Fiji Dollar', 'FJD'),
 ('Gambian Dalasi', 'GMD'),
 ('Georgian Lari', 'GEL'),
 ('Ghanaian Cedi', 'GHC'),
 ('Gibraltar Pound', 'GIP'),
 ('Gold (oz.)', 'XAU'),
 ('Guatemalan Quetzal', 'GTQ'),
 ('Guiniea Franc', 'GNF'),
 ('Guinia-Bissau Peso', 'GWP'),
 ('Guyanan Dollar', 'GYD'),
 ('Haitian Gourde', 'HTG'),
 ('Honduran Lempira', 'HNL'),
 ('Hong Kong Dollar', 'HKD'),
 ('Hungarian Forint', 'HUF'),
 ('Iceland Krona', 'ISK'),
 ('Indian Rupee', 'INR'),
 ('Indonesian Rupiah', 'IDR'),
 ('Iranian Rial', 'IRR'),
 ('Iraqi Dinar', 'IQD'),
 ('Israeli New Shekel', 'ILS'),
 ('Jamaican Dollar', 'JMD'),
 ('Japanese Yen', 'JPY'),
 ('Jordanian Dinar', 'JOD'),
 ('Kazakhstan Tenge', 'KZT'),
 ('Kenyan Schilling', 'KES'),
 ('Korean Won', 'KRW'),
 ('Kuwaiti Dinar', 'KWD'),
 ('Kyrgyzstanian Som', 'KGS'),
 ('Lao Kip', 'LAK'),
 ('Latvian Lats', 'LVL'),
 ('Lebanese Pound', 'LBP'),
 ('Lesotho Loti', 'LSL'),
 ('Liberian Dollar', 'LRD'),
 ('Libyan Dinar', 'LYD'),
 ('Lithuanian Litas', 'LTL'),
 ('Macau Pataca', 'MOP'),
 ('Macedonian Denar', 'MKD'),
 ('Malagasy Franc', 'MGF'),
 ('Malawi Kwacha', 'MWK'),
 ('Malaysian Ringgit', 'MYR'),
 ('Maldive Rufiyaa', 'MVR'),
 ('Maltese Lira', 'MTL'),
 ('Mauritanian Ouguiya', 'MRO'),
 ('Mauritius Rupee', 'MUR'),
 ('Mexican Peso', 'MXN'),
 ('Moldovan Leu', 'MDL'),
 ('Mongolian Tugrik', 'MNT'),
 ('Moroccan Dirham', 'MAD'),
 ('Mozambique Metical', 'MZM'),
 ('Myanmar Kyat', 'MMK'),
 ('Netherlands Antillian Guilder', 'ANG'),
 ('Namibian Dollar', 'NAD'),
 ('Nepalese Rupee', 'NPR'),
 ('New Zealand Dollar', 'NZD'),
 ('Nicaraguan Cordoba Oro', 'NIO'),
 ('Nigerian Naira', 'NGN'),
 ('North Korean Won', 'KPW'),
 ('Norwegian Kroner', 'NOK'),
 ('Omani Rial', 'OMR'),
 ('Pakistan Rupee', 'PKR'),
 ('Palladium (oz.)', 'XPD'),
 ('Panamanian Balboa', 'PAB'),
 ('Papua New Guinea Kina', 'PGK'),
 ('Paraguay Guarani', 'PYG'),
 ('Peruvian Neuevo Sol', 'PEN'),
 ('Philippine Peso', 'PHP'),
 ('Platinum (oz.)', 'XPT'),
 ('Polish Zloty', 'PLN'),
 ('Qatari Rial', 'QAR'),
 ('Romanian Leu', 'RON'),
 ('Russian Rouble', 'RUB'),
 ('Rwandan Franc', 'RWF'),
 ('Samoan Tala', 'WST'),
 ('Sao Tome and Principe Dobra', 'STD'),
 ('Saudi Riyal', 'SAR'),
 ('Serbian Dinar', 'RSD'),
 ('Seychelles Rupee', 'SCR'),
 ('Sierra Leone Leone', 'SLL'),
 ('Silver (oz.)', 'XAG'),
 ('Singapore Dollar', 'SGD'),
 ('Slovak Koruna', 'SKK'),
 ('Slovenian Tolar', 'SIT'),
 ('Solomon Islands Dollar', 'SBD'),
 ('Somali Schilling', 'SOS'),
 ('South African Rand', 'ZAR'),
 ('Sri Lanka Rupee', 'LKR'),
 ('St. Helena Pound', 'SHP'),
 ('Sudanese Dinar', 'SDD'),
 ('Sudanese Pound', 'SDG'),
 ('Suriname Dollar', 'SRD'),
 ('Suriname Guilder', 'SRG'),
 ('Swaziland Lilangeni', 'SZL'),
 ('Swedish Krona', 'SEK'),
 ('Swiss Franc', 'CHF'),
 ('Syrian Pound', 'SYP'),
 ('Taiwan Dollar', 'TWD'),
 ('Tanzanian Schilling', 'TZS'),
 ('Thai Baht', 'THB'),
 ('Tongan Pa''anga', 'TOP'),
 ('Trinidad and Tobago Dollar', 'TTD'),
 ('Tunisian Dinar', 'TND'),
 ('Turkish Lira', 'TRY'),
 ('Turkmenistan Manat', 'TMM'),
 ('US Dollar', 'USD'),
 ('Uganda Schilling', 'UGX'),
 ('Ukraine Hryvnia', 'UAG'),
 ('Ukraine Karbovanets', 'UAK'),
 ('United Kingdom Pound', 'GBP'),
 ('Uruguayan Peso', 'UYU'),
 ('United Arab Emirates Dirham', 'AED'),
 ('Vanuatu Vatu', 'VUV'),
 ('Venezuelan Bolivar', 'VEB'),
 ('Vietnamese Dong', 'VND'),
 ('Yugoslav Dinar', 'YUN'),
 ('Zambian Kwacha', 'ZMK'),
 ('Zimbabwe Dollar', 'ZWD');


INSERT INTO currency_rates (iso, rate, rate_date) VALUES ('AFA', 42.8, '2007-07-20'),
('AMD', 561, '2007-07-20'),
('AZM', 4908, '2007-07-20'),
('BDT', 59.6, '2007-07-20'),
('BYB', 2149.04, '2007-07-20'),
('INR', 40.2979, '2007-07-20'),
('BTN', 40.2824, '2007-07-20'),
('USD', 1, '2007-07-20'),
('BND', 1.51579, '2007-07-20'),
('KHR', 3839.6, '2007-07-20'),
('CNY', 8.10495, '2007-07-20'),
('IDR', 9032.45, '2007-07-20'),
('GEL', 2.04, '2007-07-20'),
('HKD', 7.81949, '2007-07-20'),
('JPY', 122.536, '2007-07-20'),
('KZT', 122.485, '2007-07-20'),
('KPW', 2.19688, '2007-07-20'),
('KRW', 916.676, '2007-07-20'),
('KGS', 43.75, '2007-07-20'),
('LAK', 7613.55, '2007-07-20'),
('MOP', 8.0362, '2007-07-20'),
('MYR', 3.44541, '2007-07-20'),
('MVR', 11.8425, '2007-07-20'),
('MNT', 1102.93, '2007-07-20'),
('MMK', 5.60326, '2007-07-20'),
('NPR', 75.13, '2007-07-20'),
('PKR', 60.4927, '2007-07-20'),
('PGK', 3.08019, '2007-07-20'),
('PHP', 45.792, '2007-07-20'),
('SGD', 1.51579, '2007-07-20'),
('LKR', 111.697, '2007-07-20'),
('TWD', 32.7925, '2007-07-20'),
('THB', 30.5519, '2007-07-20'),
('TMM', 5200.26, '2007-07-20'),
('UZS', 969.36, '2007-07-20'),
('VND', 14051.1, '2007-07-20'),
('DZD', 71.6047, '2007-07-20'),
('AOK', 40.7079, '2007-07-20'),
('BWP', 4.3732, '2007-07-20'),
('XOF', 475.79, '2007-07-20'),
('BIF', 1051.24, '2007-07-20'),
('XAF', 476.185, '2007-07-20'),
('CVE', 83.5055, '2007-07-20'),
('KMF', 358.034, '2007-07-20'),
('DJF', 177.721, '2007-07-20'),
('EGP', 5.78067, '2007-07-20'),
('ERN', 13.5516, '2007-07-20'),
('ETB', 8.51137, '2007-07-20'),
('GMD', 29.745, '2007-07-20'),
('GHC', 8204.8, '2007-07-20'),
('GNF', 1950.63, '2007-07-20'),
('KES', 81.2253, '2007-07-20'),
('LSL', 10.2114, '2007-07-20'),
('LRD', 1, '2007-07-20'),
('LYD', 1.34, '2007-07-20'),
('MGF', 6201, '2007-07-20'),
('MWK', 74.0891, '2007-07-20'),
('MRO', 255.016, '2007-07-20'),
('MUR', 28.4338, '2007-07-20'),
('MAD', 8.14109, '2007-07-20'),
('MZM', 23134.9, '2007-07-20'),
('NAD', 6.98032, '2007-07-20'),
('NGN', 132.296, '2007-07-20'),
('RWF', 530.45, '2007-07-20'),
('STD', 8998.21, '2007-07-20'),
('SCR', 5.44327, '2007-07-20'),
('SLL', 2372, '2007-07-20'),
('SOS', 2619.42, '2007-07-20'),
('ZAR', 6.98032, '2007-07-20'),
('SHP', 0.491585, '2007-07-20'),
('SDD', 258.207, '2007-07-20'),
('SZL', 7.85, '2007-07-20'),
('TZS', 1017.72, '2007-07-20'),
('TND', 1.27875, '2007-07-20'),
('UGX', 1908.5, '2007-07-20'),
('ZMK', 4685.84, '2007-07-20'),
('ZWD', 824, '2007-07-20'),
('BSD', 2.69389, '2007-07-20'),
('XCD', 2.69389, '2007-07-20'),
('ANG', 1.78011, '2007-07-20'),
('AWG', 1.77, '2007-07-20'),
('BBD', 2, '2007-07-20'),
('BZD', 2, '2007-07-20'),
('KYD', 0.833333, '2007-07-20'),
('CRC', 457, '2007-07-20'),
('CUP', 1, '2007-07-20'),
('DOP', 49.261, '2007-07-20'),
('SVC', 8.75, '2007-07-20'),
('GTQ', 7.91, '2007-07-20'),
('HTG', 35.9518, '2007-07-20'),
('HNL', 18.5612, '2007-07-20'),
('JMD', 61.3221, '2007-07-20'),
('NIO', 14.926, '2007-07-20'),
('PYG', 6274, '2007-07-20'),
('TTD', 6.1364, '2007-07-20'),
('ALL', 102.3, '2007-07-20'),
('ESP', 120.786, '2007-07-20'),
('FRF', 4.76185, '2007-07-20'),
('ADP', 120.786, '2007-07-20'),
('BEF', 29.2843, '2007-07-20'),
('BAM', 141.864, '2007-07-20'),
('BGL', 1.46, '2007-07-20'),
('HRK', 6.63035, '2007-07-20'),
('CZK', 20.4312, '2007-07-20'),
('DKK', 5.41578, '2007-07-20'),
('EEK', 11.3585, '2007-07-20'),
('EUR', 0.725337, '2007-07-20'),
('NOK', 5.41129, '2007-07-20'),
('FIM', 4.31624, '2007-07-20'),
('DEM', 1.41981, '2007-07-20'),
('GBP', 0.491585, '2007-07-20'),
('GRD', 247.364, '2007-07-20'),
('HUF', 178.028, '2007-07-20'),
('ISK', 60.0078, '2007-07-20'),
('ITL', 1405.61, '2007-07-20'),
('LVL', 0.517215, '2007-07-20'),
('LTL', 2.56516, '2007-07-20'),
('MTL', 0.362024, '2007-07-20'),
('MDL', 12.2928, '2007-07-20'),
('PLN', 2.72018, '2007-07-20'),
('ROL', 26002, '2007-07-20'),
('RON', 2.6002, '2007-07-20'),
('RUB', 25.5012, '2007-07-20'),
('SKK', 24.0741, '2007-07-20'),
('SIT', 182.122, '2007-07-20'),
('SEK', 6.64243, '2007-07-20'),
('CHF', 1.20207, '2007-07-20'),
('TRY', 1.27756, '2007-07-20'),
('UAG', 5.05022, '2007-07-20'),
('BMD', 1, '2007-07-20'),
('CAD', 1.04605, '2007-07-20'),
('MXN', 10.7817, '2007-07-20'),
('FJD', 1.57268, '2007-07-20'),
('AUD', 1.15746, '2007-07-20'),
('NZD', 1.27258, '2007-07-20'),
('WST', 3.34102, '2007-07-20'),
('SBD', 7.095, '2007-07-20'),
('TOP', 1.7043, '2007-07-20'),
('VUV', 107.057, '2007-07-20'),
('XPF', 86.5072, '2007-07-20'),
('BHD', 0.377, '2007-07-20'),
('CYP', 0.461023, '2007-07-20'),
('IRR', 8200, '2007-07-20'),
('ILS', 4.27308, '2007-07-20'),
('JOD', 0.709145, '2007-07-20'),
('KWD', 0.295014, '2007-07-20'),
('LBP', 1509.4, '2007-07-20'),
('OMR', 0.384989, '2007-07-20'),
('QAR', 3.64023, '2007-07-20'),
('SAR', 3.75016, '2007-07-20'),
('SYP', 46.4305, '2007-07-20'),
('AED', 3.6745, '2007-07-20'),
('YER', 176.304, '2007-07-20'),
('ARS', 3.11375, '2007-07-20'),
('BOB', 7.9908, '2007-07-20'),
('BRL', 1.87693, '2007-07-20'),
('CLP', 516.244, '2007-07-20'),
('COP', 2320, '2007-07-20'),
('ECS', 25000, '2007-07-20'),
('GYD', 179.534, '2007-07-20'),
('PEN', 3.30516, '2007-07-20'),
('SRG', 2465.07, '2007-07-20'),
('UYU', 26.7417, '2007-07-20'),
('VEB', 2146.99, '2007-07-20');

INSERT INTO currency_weight (iso, weight) VALUES ('AUD', 0.7),
 ('CHF', 2.6),
 ('EUR', 41.3),
 ('GBP', 11.1),
 ('HKD', 2.1),
 ('JPY', 13.6),
 ('SEK', 1.1),
 ('SGD', 0.5),
 ('USD', 24.3);

INSERT INTO thresholds (iso, threshold) VALUES ('AUD', 23149.2),
 ('USD', 20000),
 ('SEK', 132849),
 ('SGD', 30315.8),
 ('EUR', 14506.7),
 ('JPY', 2.45072e+06),
 ('HKD', 156390),
 ('CHF', 24041.5),
 ('GBP', 9831.7),
 ('XAF', 9.5237e+06),
 ('HUF', 3.56055e+06),
 ('AFA', 856000),
 ('IDR', 1.80649e+08),
 ('PKR', 1.20985e+06),
 ('ZWD', 1.648e+07),
 ('CVE', 1.67011e+06),
 ('SOS', 5.23885e+07),
 ('BBD', 40000),
 ('DOP', 985220),
 ('CAD', 20921);